"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const request_promise_1 = __importDefault(require("request-promise"));
const https_1 = require("https");
const helpers_1 = require("../helpers");
// tslint:disable-next-line: no-var-requires
const CFError = require('cf-errors');
const logger = helpers_1.createNewLogger('codefresh:infra:git-providers:bitbucket-server');
const LIMIT_PER_PAGE = 500;
const MAX_PAGES = 20;
const ApiVersions = {
    V1: 'rest/api/1.0/',
};
const COMMIT_METADATA_PATH = 'com.atlassian.bitbucket.server.bitbucket-branch:latest-commit-metadata';
const _cleanEncodedFilePath = (filepath) => {
    const prefix = './';
    const hasPrefix = filepath.indexOf(prefix) === 0;
    const path = hasPrefix ? filepath.substring(prefix.length) : filepath;
    return encodeURIComponent(path);
};
const _extractErrorFromResponse = (response) => {
    let errStr = `${lodash_1.get(response, 'body.errors[0].message')}`;
    if (lodash_1.get(response, 'body.errors[0].exceptionName')) {
        errStr = `${errStr} [${lodash_1.get(response, 'body.errors[0].exceptionName')}]`;
    }
    return errStr;
};
const _toBranch = (rawBranch) => ({
    name: rawBranch.displayId,
    id: rawBranch.id,
    commit: {
        sha: rawBranch.metadata[COMMIT_METADATA_PATH].id,
        commiter_name: rawBranch.metadata[COMMIT_METADATA_PATH].author.name,
        message: rawBranch.metadata[COMMIT_METADATA_PATH].message,
        url: '',
    }
});
class BitbucketServer {
    constructor(opts) {
        if (!opts.apiURL) {
            throw new CFError('Cannot create bitbucket-server provider without api url');
        }
        this.baseUrl = opts.apiURL;
        if (!this.baseUrl.endsWith('/')) {
            this.baseUrl = `${this.baseUrl}/`;
        }
        this.timeout = opts.timeout || 10000;
        if (opts.type === 'oauth2') {
            this.authenticationHeader = { Authorization: `Bearer ${opts.password}` };
        }
        else {
            this.authenticationHeader = { Authorization: `Basic ${Buffer.from(`${opts.username}:${opts.password}`).toString('base64')}` };
        }
        if (process.env.NODE_TLS_REJECT_UNAUTHORIZED === '0' || opts.insecure) {
            logger.warn('using insecure mode');
            this.agent = new https_1.Agent({ rejectUnauthorized: false });
        }
    }
    async performAPICall(opts) {
        const method = opts.method || 'GET';
        const requestHeaders = lodash_1.merge(opts.noAuth ? {} : this.authenticationHeader, opts.headers || {});
        const requestOptions = {
            method,
            url: `${this.baseUrl}${opts.apiVersion || ApiVersions.V1}${opts.api}`,
            qs: opts.qs || {},
            headers: requestHeaders,
            json: opts.json,
            timeout: this.timeout,
            form: opts.data,
            resolveWithFullResponse: true,
            agent: this.agent,
            simple: false,
        };
        logger.debug(`${method} ${requestOptions.url} qs: ${JSON.stringify(requestOptions.qs)}`);
        return request_promise_1.default(requestOptions)
            .then((res) => {
            const curLogger = res.statusCode >= 400 ? logger.error.bind(logger) : logger.debug.bind(logger);
            curLogger(`${method} ${requestOptions.url} qs: ${JSON.stringify(requestOptions.qs)} status: ${res.statusCode}`);
            return res;
        });
    }
    async paginateForResult(opts, predicate) {
        let start = 0;
        let page = 0;
        let isLastPage = false;
        while (!isLastPage && page < MAX_PAGES) {
            opts.qs = lodash_1.merge(opts.qs, {
                start: '' + start,
                limit: '' + LIMIT_PER_PAGE,
            });
            const res = await this.performAPICall(opts);
            if (res.statusCode >= 400) {
                throw new CFError(`Failed with status code: ${res.statusCode}, ${_extractErrorFromResponse(res)}`);
            }
            isLastPage = res.body.isLastPage;
            const predicateResult = lodash_1.find(res.body.values, predicate);
            if (predicateResult) {
                return predicateResult; // found the wanted result
            }
            start += LIMIT_PER_PAGE;
            page += 1;
        }
        return; // no result found
    }
    async getProjectKey(projectName) {
        const [getProjErr, proj] = await helpers_1.to(this.paginateForResult({
            api: `projects`,
            json: true,
            qs: {
                name: projectName,
            }
        }, project => project.name === projectName));
        if (getProjErr) {
            throw new CFError(`Failed to get project "${projectName}", with: ${getProjErr}`);
        }
        if (!proj) {
            throw new CFError(`Project with name "${projectName}" was not found. To use the user-centric api add the "~" prefix: "~${projectName}"`);
        }
        return proj.key;
    }
    getName() {
        return 'bitbucket-server';
    }
    async fetchRawFile(opts) {
        const projectKey = opts.owner.startsWith('~') ? opts.owner : await this.getProjectKey(opts.owner);
        const lines = [];
        let start = 0;
        let page = 0;
        let isLastPage = false;
        while (!isLastPage && page < MAX_PAGES) {
            const res = await this.performAPICall({
                apiVersion: ApiVersions.V1,
                api: `projects/${projectKey}/repos/${opts.repo}/browse/${_cleanEncodedFilePath(opts.path)}`,
                json: true,
                qs: {
                    at: opts.ref,
                    raw: 'true',
                    start: '' + start,
                    limit: '' + LIMIT_PER_PAGE,
                }
            });
            if (res.statusCode >= 400) {
                throw new CFError(`Failed to retrieve file: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
            }
            isLastPage = res.body.isLastPage;
            lines.push(...lodash_1.map(res.body.lines, (line) => line.text));
            start += LIMIT_PER_PAGE;
            page += 1;
        }
        return lines.join('\n');
    }
    async getBranch(opts) {
        const projectKey = opts.owner.startsWith('~') ? opts.owner : await this.getProjectKey(opts.owner);
        const [getBranchErr, branch] = await helpers_1.to(this.paginateForResult({
            apiVersion: ApiVersions.V1,
            api: `projects/${projectKey}/repos/${opts.repo}/branches`,
            json: true,
            qs: {
                filterText: opts.branch,
                details: 'true',
            }
        }, (b) => lodash_1.isEqual(opts.branch, b.displayId ? b.displayId : lodash_1.last(b.id.split('/')))));
        if (getBranchErr) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opts)}, ${getBranchErr}`);
        }
        if (!branch) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opts)}, branch not found`);
        }
        return _toBranch(branch);
    }
    async getRepository(opt) {
        throw new Error("Method not implemented.");
    }
    async listBranches(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOwner(opt) {
        throw new Error("Method not implemented.");
    }
    async createRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listWebhooks(opt) {
        throw new Error("Method not implemented.");
    }
    async deleteRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesWithAffiliation(opt) {
        throw new Error("Method not implemented.");
    }
    async listOrganizations(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOrganization(opt) {
        throw new Error("Method not implemented.");
    }
    async createCommitStatus(opt) {
        throw new Error("Method not implemented.");
    }
}
exports.default = BitbucketServer;
//# sourceMappingURL=bitbucket-server.js.map