"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const request_promise_1 = __importDefault(require("request-promise"));
const https_1 = require("https");
// tslint:disable-next-line: no-var-requires
const CFError = require('cf-errors');
const ApiVersions = {
    V1: 'api/1.0/',
    V2: 'api/2.0/',
    OAUTH: 'site/oauth2/access_token',
};
const _cleanEncodedFilePath = (filepath) => {
    const prefix = './';
    const hasPrefix = filepath.indexOf(prefix) === 0;
    const path = hasPrefix ? filepath.substring(prefix.length) : filepath;
    return encodeURIComponent(path);
};
class Bitbucket {
    constructor(opts) {
        this.baseUrl = opts.apiURL || 'https://bitbucket.org/';
        if (!this.baseUrl.endsWith('/')) {
            this.baseUrl = `${this.baseUrl}/`;
        }
        this.timeout = opts.timeout || 10000;
        if (opts.type === 'oauth2') {
            this.authenticationHeader = { Authorization: `Bearer ${opts.password}` };
        }
        else {
            this.authenticationHeader = { Authorization: `Basic ${Buffer.from(`${opts.username}:${opts.password}`).toString('base64')}` };
        }
        if (process.env.NODE_TLS_REJECT_UNAUTHORIZED === '0' || opts.insecure) {
            this.agent = new https_1.Agent({ rejectUnauthorized: false });
        }
    }
    getName() {
        return 'bitbucket';
    }
    async performAPICall(opts) {
        const method = opts.method || 'GET';
        const requestHeaders = lodash_1.default.merge(opts.noAuth ? {} : this.authenticationHeader, opts.headers || {});
        const requestOptions = {
            method,
            url: `${this.baseUrl}${opts.apiVersion}${opts.api}`,
            qs: opts.qs || {},
            headers: requestHeaders,
            json: opts.json,
            timeout: this.timeout,
            form: opts.data,
            resolveWithFullResponse: true,
            agent: this.agent,
        };
        return request_promise_1.default(requestOptions);
    }
    async fetchRawFile(opts) {
        const res = await this.performAPICall({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}/src/${opts.ref}/${_cleanEncodedFilePath(opts.path)}`,
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to retrieve file: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
        }
        return res.body;
    }
    async getBranch(opts) {
        const res = await this.performAPICall({
            apiVersion: ApiVersions.V2,
            api: `repositories/${opts.owner}/${opts.repo}/refs/branches/${opts.branch}`,
            json: true
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opts)}, status code: ${res.statusCode}`);
        }
        return {
            name: res.body.name,
            id: res.body.name,
            commit: {
                sha: res.body.target.hash,
                commiter_name: lodash_1.default.get(res.body, 'target.author.user.display_name', ''),
                message: lodash_1.default.get(res.body, 'target.message', ''),
                url: lodash_1.default.get(res.body, 'target.links.html.href', ''),
            }
        };
    }
    async getRepository(opt) {
        throw new Error("Method not implemented.");
    }
    async listBranches(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOwner(opt) {
        throw new Error("Method not implemented.");
    }
    async createRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listWebhooks(opt) {
        throw new Error("Method not implemented.");
    }
    async deleteRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesWithAffiliation(opt) {
        throw new Error("Method not implemented.");
    }
    async listOrganizations(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOrganization(opt) {
        throw new Error("Method not implemented.");
    }
    async createCommitStatus(opt) {
        throw new Error("Method not implemented.");
    }
}
exports.default = Bitbucket;
//# sourceMappingURL=bitbucket.js.map