"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const request_promise_1 = __importDefault(require("request-promise"));
const https_1 = require("https");
// tslint:disable-next-line: no-var-requires
const CFError = require('cf-errors');
const _encodeProjectPath = (owner, repo) => {
    return encodeURIComponent(`${owner}/${repo}`).replace(/\./g, '%2E');
};
const _cleanEncodedFilePath = (filepath) => {
    const prefix = './';
    const hasPrefix = filepath.indexOf(prefix) === 0;
    const path = hasPrefix ? filepath.substring(prefix.length) : filepath;
    return encodeURIComponent(path);
};
class Gitlab {
    constructor(opts) {
        this.baseUrl = opts.apiURL || 'https://gitlab.com/api/v4/';
        if (!this.baseUrl.endsWith('/')) {
            this.baseUrl = `${this.baseUrl}/`;
        }
        this.timeout = opts.timeout || 10000;
        // works for both oauth2 and private token authentication methods
        this.authenticationHeader = opts.password ? { Authorization: `Bearer ${opts.password}` } : {};
        if (process.env.NODE_TLS_REJECT_UNAUTHORIZED === '0' || opts.insecure) {
            this.agent = new https_1.Agent({ rejectUnauthorized: false });
        }
    }
    async performAPICall(opts) {
        const method = opts.method || 'GET';
        const requestHeaders = lodash_1.default.merge(opts.noAuth ? {} : this.authenticationHeader, opts.headers || {});
        const requestOptions = {
            method,
            url: `${this.baseUrl}${opts.api}`,
            qs: opts.qs || {},
            headers: requestHeaders,
            json: opts.json,
            timeout: this.timeout,
            form: opts.data,
            resolveWithFullResponse: true,
            agent: this.agent,
        };
        return request_promise_1.default(requestOptions);
    }
    getName() {
        return 'gitlab';
    }
    async fetchRawFile(opt) {
        const res = await this.performAPICall({
            api: `projects/${_encodeProjectPath(opt.owner, opt.repo)}/repository/files/${_cleanEncodedFilePath(opt.path)}`,
            qs: { ref: opt.ref },
            json: true,
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to retrieve file: ${JSON.stringify(opt)}, status code: ${res.statusCode}`);
        }
        const base64Content = res.body.content.replace(/\n/ig, '');
        return Buffer.from(base64Content, 'base64').toString();
    }
    async getBranch(opt) {
        const res = await this.performAPICall({
            api: `projects/${_encodeProjectPath(opt.owner, opt.repo)}/repository/branches/${opt.branch}`,
            json: true
        });
        if (res.statusCode >= 400) {
            throw new CFError(`Failed to retrieve branch: ${JSON.stringify(opt)}, status code: ${res.statusCode}`);
        }
        return {
            name: res.body.name,
            id: res.body.name,
            commit: {
                sha: res.body.commit.id,
                commiter_name: res.body.commit.author_name,
                message: res.body.commit.message,
                url: res.body.commit.web_url,
            }
        };
    }
    async getRepository(opt) {
        throw new Error("Method not implemented.");
    }
    async listBranches(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOwner(opt) {
        throw new Error("Method not implemented.");
    }
    async listWebhooks(opt) {
        throw new Error("Method not implemented.");
    }
    async createRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async deleteRepositoryWebhook(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesWithAffiliation(opt) {
        throw new Error("Method not implemented.");
    }
    async listOrganizations(opt) {
        throw new Error("Method not implemented.");
    }
    async listRepositoriesForOrganization(opt) {
        throw new Error("Method not implemented.");
    }
    async createCommitStatus(opt) {
        throw new Error("Method not implemented.");
    }
}
exports.default = Gitlab;
//# sourceMappingURL=gitlab.js.map